#!/usr/bin/env Rscript
############################################################
# GENERATING FOREST PLOTS FOR pQTL MRs
############################################################

library(dplyr)
library(forestplot)

############################################################
# LOADING DATA
############################################################

setwd('../MR_data')
getwd()

# the relevant data
content <- c("CETP_HDLC.csv", "CETP_LDLC.csv", "CETP_pQTL.csv",
    "CETP_TG.csv", "PCSK9_LDLC.csv", "PCSK9_pQTL.csv")

# functions
source('../scripts/helper_code/Premable.R')
source("../scripts/helper_code/MR_forestplots.R")

# output
output = '../figures/'

# Data for Figure 3
CETP_input <- list(
    read_delim('Figure3_CETP_estimates.tsv', delim = '\t'),
    read_delim('Figure3_CETP_counts.tsv', delim = '\t'),
    c( TRUE,TRUE,TRUE,FALSE,FALSE,FALSE,FALSE,FALSE,FALSE,FALSE,FALSE,TRUE,
    TRUE,FALSE,FALSE,FALSE,FALSE,FALSE,FALSE,FALSE,FALSE,FALSE,FALSE,FALSE
)
)

PCSK9_input <- list(
    read_delim('Figure3_PCSK9_estimates.tsv', delim = '\t'),
    read_delim('Figure3_PCSK9_counts.tsv', delim = '\t'),
    c( TRUE,TRUE,TRUE,FALSE,FALSE,FALSE,FALSE,FALSE,FALSE,FALSE,FALSE,TRUE,
    TRUE,FALSE,FALSE,FALSE,FALSE,FALSE,FALSE,FALSE,FALSE,FALSE,FALSE,FALSE
)
)

# Constants 
w <- 16 * 0.393700787
h <- 14 * 0.393700787
############################################################
# Figure 3
############################################################

pdf(paste0(output,'Figure3A.pdf'), width =w , height = h, compress = F, onefile=T)

forest(CETP_input, xlab = expression(paste('Decrease in CETP')), xlim = c(0.8, 1.5), at = c(0.8, 1, 1.4))
grid.text('CETP', just = 'left', .03, .95, gp=gpar(cex=1.5, fontface=4))

dev.off()

pdf(paste0(output,'Figure3B.pdf'), width =w , height = h, compress = F, onefile=T)

forest(PCSK9_input, xlab = expression(paste('Decrease in PCSK9')), xlim = c(0.5, 3), at = c(0.5, 1, 3))
grid.text('PCSK9', just = 'left', .03, .95, gp=gpar(cex=1.5, fontface=4))

dev.off()
