#!/usr/bin/env Rscript

############################################################
# GENERATING FOREST PLOTS FOR pQTL MRs
############################################################

source("helper_code/Premable.R")
source("helper_code/MR_forestplots.R")
source("helper_code/Extending_the_WesPalette.R")
source("helper_code/CharRound.R")
source("helper_code/ScientificNotation.R")
library(dplyr)
library(forestplot)
library(openxlsx)
library(ggsignif)

############################################################
# LOADING DATA
############################################################

file = '../TRIAL_data/CETPtrials.xlsx'
bin_fe <- read.xlsx(file, sheet='BIN_FE_MA', rowNames =T)
con_fe <- read.xlsx(file, sheet='CON_FE_MA', rowNames =T)
bin_re <- read.xlsx(file, sheet='BIN_ME_MA', rowNames =T)
con_re <- read.xlsx(file, sheet='CON_ME_MA', rowNames =T)

col1 <- getWesPalette('FantasticFox1', n = 5, m = 5)[c(2:5)]

# output 
output <- '../figures/'

# ---- ggplot defaults
theme_update(
            plot.title = element_text(size = 10, face = "bold"), # should you want a plot title
            plot.subtitle = element_text(size = 6, face = "italic"),
            panel.background = element_blank(), # legend.title
            panel.grid.major = element_line(size = (0.3), colour="grey"),
            axis.line = element_line(colour = "black"),
            axis.title=element_text(size=10,face="bold"),
            axis.text=element_text(size=5.4),
            legend.background = element_rect(fill = "transparent"), # get rid of legend bg
            legend.key = element_blank(),
            legend.title = element_text(size = 6),
            legend.text = element_text(size = 5),
            legend.key.size = unit(0.2, "cm"), # size grey box size)
            strip.background = element_rect(fill = "transparent")
)

###########################################################
# BARPLOT for CONTIN DATA
###########################################################

QC_CON <- function(input){
    # removing overall
    # input <- con_fe
    r1 <- grep('Overall', rownames(input))
    con_input <-  input[-r1,]

    # remove mmol/L measurments (retaining percentage change)
    r2 <- grep('mmol/L|mg/dL|diabetics', rownames(con_input))
    con_input <- con_input[-r2,]

    # outcome factor
    row1 <- rownames(con_input)
    con_input$biomarker <- sub('\\s*\\([^\\)]+\\)' ,'', sub("^[A-Z]*[a-z]* ", "", row1))
    # con_input$biomarker <- sub('\\s*\\([^\\)]+\\)' ,'', sub(".* ", "", row1))
    con_input$biomarker <- ifelse(con_input$biomarker == 'TRI', 'TG', con_input$biomarker)
    con_input$biomarker <- factor(con_input$biomarker,
        levels = c('HDL', 'LDL', 'TG', 'ApoA1', 'ApoB', 'LPA', 'SBP', 'DBP'),
        labels = c('HDL-C', 'LDL-C', 'TG', 'Apoa', 'Apob','Lp[a]', 'SBP','DBP'))

    # Compound factor
    con_input$compound <- sub(" .*", "", row1)
    con_input$compound <- factor(con_input$compound,
        levels = c("Anacetrapib", "Evacetrapib", "Dalecetrapib", "Torcetrapib"),
        labels = c("Anacetrapib", "Evacetrapib", "Dalcetrapib", "Torcetrapib"))

    # heterogeneity p-values
    con_input$p.value <- con_input$Qpval

    #return
    return(con_input)
}

con_input <- QC_CON(con_fe)

# seperate barplots
bar_in1 <- con_input[!con_input$biomarker %in% c('SBP','DBP'),]
bar_in2 <- con_input[con_input$biomarker %in% c('SBP','DBP'),]

# correcting an mistake (delete once repaired at the source)
bar_in1[,c("Point", "SE", "LB", "UB")] <-
bar_in1[,c("Point", "SE", "LB", "UB")] * 100
ylim1 <- c(floor(min(bar_in1$Point, na.rm=T)), ceiling(max(bar_in1$Point, na.rm=T))*1.12)
ylim2 <- c(floor(min(bar_in2$Point, na.rm=T)), ceiling(max(bar_in2$Point, na.rm=T))*1.12)

barplot_g <- function(input, col = col1){
    ggplot(input, aes(fill=compound, y=Point, x = biomarker)) +
        geom_bar(position='dodge', stat='identity', col = 'black', size = 0.3) +
        geom_errorbar(position=position_dodge(width = 0.9), aes(x=biomarker, ymin=LB, ymax=UB),
        , width=0.3) +
        geom_point(position=position_dodge(width = 0.9)) +
        scale_fill_manual(values = col) +
        theme(
            plot.margin = unit(c(0.2,0.0,0.2,0.2), "lines")
        )

}


bar1 <- barplot_g(bar_in1) +
        labs(x = 'Lipids', y = 'Percentage change') +
        theme(legend.position = "none") + coord_cartesian(ylim =ylim1)
bar1 <- bar1 + geom_signif(xmin=c(0.6, 1.6, 2.6, 3.6, 4.6, 5.6),
    xmax = c(1.4, 2.4, 3.4, 4.4, 5.4,6.4), y_position = c(140, 10, 10, 57, 32, 10),
    annotations = c(rep('***', 2), '***', rep('***', 2), '**'), size = 0.3, textsize = 3)
    # annotations = c(rep('***', 6)), size = 0.3, textsize = 3)
bar2 <- barplot_g(bar_in2) +
        labs(x = 'Blood pressure', y = 'mmHg') + coord_cartesian(ylim =ylim2) +
        guides(fill=guide_legend(title="Compounds")) +
        theme(legend.key.size = unit(0.4, "cm"))
# bar2 <- ggplot(bar_in2, aes(fill=compound, y=Point, x = biomarker)) +
#     geom_bar(position='dodge', stat='identity', col = 'black') +
#     geom_errorbar(position=position_dodge(width = 0.9), aes(x=biomarker, ymin=LB, ymax=UB),
#     , width=0.3, show.legend=FALSE) +
#     geom_point(position=position_dodge(width = 0.9), show.legend=FALSE) +
#     scale_fill_manual(values = col1) +
bar2 <- bar2 + geom_signif(xmin=c(0.6, 1.6), xmax = c(1.4, 2.4), y_position = c(5.0, 2.5),
    annotations = '***', size = 0.3, textsize = 3)

############################################################
# FOREST for BINARY data
############################################################

QC_BIN <- function(input){
    # removing overall
    r3 <- grep('Overall', rownames(input))
    bin_input <-  input[-r3,]

    # remove outcomes
    r4 <- grep('CR|AE|SAE|SBP|DBP|CK|KIDNEY|UTI|T2DM', rownames(bin_input))
    bin_input <- bin_input[-r4,]

    # outcome factor
    row3 <- rownames(bin_input)
    bin_input$outcome <- sub(".* ", "", row3)
    bin_input$outcome <- factor(bin_input$outcome,
        levels = c('ACM', 'ACVD', 'FCVD', 'AMI', 'FMI', 'AST', 'IST', 'HST',
        'HF'),
        labels = c('ACM', 'CVD', 'FCVD', 'MI', 'FMI', 'ST',
        'IST', 'HST', 'HF'))

    # Compound factor
    bin_input$compound <- sub(" .*", "", row3)
    bin_input$compound <- factor(bin_input$compound,
        levels = c("Anacetrapib", "Evacetrapib", "Dalecetrapib", "Torcetrapib"),
        labels = c("Anacetrapib", "Evacetrapib", "Dalcetrapib", "Torcetrapib"))

    # heterogeneity p-values
    bin_input$p.value <- bin_input$Qpval

    # formatting
    # bin_input[,c('Point','LB', 'UB')] <- exp(bin_input[,c('Point','LB', 'UB')])
    bin_input[,c('Point','LB', 'UB')] <- (bin_input[,c('Point','LB', 'UB')])
    bin_input$ET <- paste0(bin_input$Nevents, '/', bin_input$NumObs)
    bin_input$ORCI <- paste0(CharRound(bin_input$Point), ' (', CharRound(bin_input$LB), ';',
        CharRound(bin_input$UB), ')')
    bin_input$ORCI[bin_input$ORCI == 'NA (NA;NA)'] <- 'No data'
    bin_input$het <- ifelse(bin_input$Qpval < 0.001, '< 0.001', CharRound(bin_input$Qpval,3))
    bin_input[-seq(1,nrow(bin_input)-3, 4),'het'] <- ''

    # return
    return(bin_input)
}

bin_input <- QC_BIN(bin_fe)

# forestplot
forplot_g <- function(input, col = col1){
    ggplot(data=input,
        aes(x = compound,y = Point, ymin = LB, ymax = UB ))+
        geom_hline(aes(fill=Group),yintercept =1, linetype=2)+
        geom_pointrange(aes(col=compound), size = 0.2) +
        geom_errorbar(aes(ymin=LB, ymax=UB,col=compound),width=0.5,cex=0.5)+
        facet_wrap(~outcome,strip.position="left",nrow=9,scales = "fixed") +
        scale_colour_manual(values = col1) +
        theme(plot.title=element_text(size=16,face="bold"),
            axis.text.y=element_blank(),
            axis.ticks.y=element_blank(),
            axis.text.x=element_text(face="bold"),
            # axis.title=element_text(size=12,face="bold"),
            strip.text.y = element_text(hjust=0,vjust = 1,angle=180,face="bold", size = 6),
            legend.position='none',
            plot.margin = unit(c(1,0,0,0), "lines")
                    )
            # scale_x_discrete(limits = rev(levels(bin_input$compound)))+
}

p <- forplot_g(bin_input) + xlab('')+ ylab("OR (95%CI)") +
        coord_flip(ylim = c(0.5, 2.1))


lab <- data.frame(V0 = rep(bin_input$compound, 2),
                    V05 = factor(rep(c('OR(95%CI)','Heterogeneity'), each = nrow(bin_input)),
                    levels = c('OR(95%CI)', 'Heterogeneity')),
                    V1 = c(bin_input$ORCI, bin_input$het),
                    outcome = rep(bin_input$outcome,2)
                )


sidetable_g <- function(input){
    ggplot(input, aes(x = V05, y = V0, label = base::format(V1, nsmall = 1))) +
    geom_text(size = 1.6, hjust=0, vjust=0.5) + theme_bw() +
    # geom_hline(aes(yintercept=c(6.5,7.5))) +
    theme(panel.grid.major = element_blank(),
        legend.position = "none",
        strip.background = element_rect(fill = "transparent"), # get rid of legend bg
        strip.text.y = element_blank(),
        strip.text.x = element_text(hjust=0,vjust = 1,angle=0,face="bold", size = 5),
        # strip.background = element_blank(),
        panel.border = element_blank(),
        axis.text.x = element_blank(),
        axis.text.y = element_blank(),
        axis.ticks = element_line(colour="white"),#element_blank(),
        plot.margin = unit(c(0.0,0,0.5,0), "lines")) +
    labs(x="",y="") +
    # scale_x_discrete(limits = rev(levels(lab$V05)))+
    coord_cartesian(xlim=c(0,20.5)) +
    facet_grid(vars(outcome), vars(V05))
    # facet_wrap(~outcome,strip.position="left",nrow=9,scales = "free_y")
}

data_table <- sidetable_g(lab)

# gridExtra::grid.arrange(data_table, p, ncol=2)
bars <- cowplot::plot_grid(bar1, bar2, nrow = 2)
fplot <- cowplot::plot_grid(p, data_table,  rel_widths = c(20,25))
cowplot::plot_grid(bars, fplot, rel_widths=c(40, 55))
H <- 10
W <- 12
ggsave(paste0(output,'Figure1.pdf'), height = H, width = W, units = 'cm')
