#!/usr/bin/env Rscript

library(forestplot)
library(dplyr)

# Helper
PointAndCI <-
function (p, s, a = 0.05, r = 2, exp = "Yes")
{
    sf <- paste0("%.", r, "f")
    if (exp == "Yes") {
        return(paste0(sprintf(sf, round(exp(p), r)), " (", sprintf(sf,
            round(exp(p - s * qnorm(1 - a/2)), r)), ";", sprintf(sf,
            round(exp(p + s * qnorm(1 - a/2)), r)), ")"))
    }
    else {
        return(paste0(sprintf(sf, round(p, r)), " (", sprintf(sf,
            round(p - s * qnorm(1 - a/2), r)), ";", sprintf(sf,
            round(p + s * qnorm(1 - a/2), r)), ")"))
    }
}

#%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  Formatting function
format <- function(in1, in2,
                    top_row = 3, mid_row = 2,
                    top_lab = matrix(c('','','','Outcome', 'Events/Total','OR(95%CI)',
                                        'CVD', '', ''), ncol = 3, nrow = 3, byrow = T),
                    mid_lab = matrix(c('','','','Non-CVD','',''), ncol = 3, nrow = 2, byrow = T),
                    p = 'point', se = 'se', selef = c('point','lb','ub'),
                    selno = c('no_cases', 'samplesize')
){
    
    # data prepration
    first <- as.matrix(in1[,selef])
    second <- as.matrix(in2[,selef])
    colnames(second) <- colnames(first) <- NULL
    
    # generating input data
    data_fig <- rbind(
                matrix(NA, ncol = 3, nrow = top_row),
                exp(first),
                matrix(NA, ncol = 3, nrow = mid_row),
                exp(second)
            )
    
    # generating input table
    table_fig <- rbind(top_lab,
                    cbind(
                        in1$outcome,
                        paste0(unlist(in1[,selno[1]]),'/', unlist(in1[,selno[2]])),
                        PointAndCI(unlist(in1[,p]), unlist(in1[,se]))
                    ),
                    mid_lab,
                    cbind(
                    in2$outcome,
                    paste0(unlist(in2[,selno[1]]),'/', unlist(in2[,selno[2]])),
                    PointAndCI(unlist(in2[,p]), unlist(in2[,se]))
                    )
                )
    # removing NA's
    selna <- table_fig[,3] == 'NA (NA;NA)'
    table_fig[selna, 3] <- rep('No data', sum(selna))
                
    # bold or not
    bold <- c(rep(TRUE, top_row), rep(FALSE, dim(in1)[1]), rep(TRUE, mid_row),
            rep(FALSE, dim(in2)[2]))
    results <-
    return(
        list(
            data_fig,
            table_fig,
            bold
    ))
}

#%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  Forestplotting function
forest <- function(list,
                    pos = 3,
                    boxsize = 0.3,
                    at = c(0.7, 0.8, 1.0, 1.2, 1.4),
                    xlim = c(0.5, 2.0),
                    xlab,
                    ...
                ){

forestplot(list[[2]],
           list[[1]],
           is.summary=list[[3]],
           clip=xlim,
           graph.pos = pos,
           xlog=TRUE,
           boxsize = boxsize,
           xticks=at,
           new_page = FALSE,
           align=c('l','r','r'),
           col=fpColors(box="black",line="black", summary="black"),
           xlab = xlab,
           txt_gp = fpTxtGp(
               ticks = gpar(cex = 1.0),
               xlab = gpar(cex = 1.0)),
               ...
       )
    
}
